//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//	ClientUDPSocket.cpp : implementation file

#include "stdafx.h"
#include "emule.h"
#include "ClientUDPSocket.h"
#include "opcodes.h"
#include "otherfunctions.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif

#define	ERR_UDP_MISCONFIGURED _T("Error while processing incoming UDP Packet")

CClientUDPSocket::CClientUDPSocket()
{
	m_bWouldBlock = false;
}

CClientUDPSocket::~CClientUDPSocket()
{
}

void CClientUDPSocket::OnReceive(int nErrorCode)
{
	EMULE_TRY

	byte 		abyteBuffer[5000];
	SOCKADDR_IN sockAddr = {0};
	int 			iSockAddrLen = sizeof(sockAddr);
	int 			iLength = ReceiveFrom(abyteBuffer, sizeof(abyteBuffer), (SOCKADDR*)&sockAddr, &iSockAddrLen);

	if (g_eMuleApp.m_pIPFilter->IsFiltered(sockAddr.sin_addr.S_un.S_addr))
	{
		InterlockedIncrement(&g_eMuleApp.m_lIncomingFiltered);
		InterlockedIncrement(&g_eMuleApp.m_lTotalFiltered);
	}
	else if (iLength >= 2 && abyteBuffer[0] == OP_EMULEPROT)
	{
		ProcessPacket(abyteBuffer + 2, iLength - 2, abyteBuffer[1], sockAddr.sin_addr.S_un.S_addr, ntohs(sockAddr.sin_port));
	}
	else if (iLength == SOCKET_ERROR)
	{
	}

	EMULE_CATCH
}

bool CClientUDPSocket::ProcessPacket(byte* pbytePacketBuf, uint16 size, byte opcode, uint32 dwIP, uint16 port)
{
	try
	{
		switch (opcode)
		{
			case OP_REASKFILEPING:
			{
				g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadFileRequest(size);
			//	Check pbytePacketBuf size
				if (size < 16)
				{
					ASSERT (false);
					break;
				}

			//	Look up the requested file hash in our shared files list.
				CKnownFile		*m_pReqPartFile = g_eMuleApp.m_pSharedFilesList->GetFileByID(reinterpret_cast<BYTE*>(pbytePacketBuf));

			//	If it's not there...
				if (m_pReqPartFile == NULL)
				{
					Packet		*pResponsePacket = new Packet(OP_FILENOTFOUND, 0, OP_EMULEPROT);

					g_eMuleApp.m_pUploadQueue->AddUpDataOverheadFileRequest(pResponsePacket->m_dwSize);
					SendPacket(pResponsePacket, dwIP, port);
					break;
				}

				CUpDownClient	*pClient = g_eMuleApp.m_pUploadQueue->GetWaitingClientByIPAndUDPPort(dwIP, port);

			//	If the client is waiting in our upload queue...
				if (pClient != NULL)
				{
				//	KuSh: Why are we setting its UDP port here ?
					pClient->SetUDPPort(port);
					pClient->AddAskedCount();
					pClient->SetLastUpRequest();
					pClient->SetLastL2HACExecution();

					if ((pClient->GetUDPVersion() > 3) && (size >= 20))
					{
						pClient->ProcessUpFileStatus(reinterpret_cast<CHAR*>(pbytePacketBuf), size);
					}
					else if ((pClient->GetUDPVersion() > 2) && (size >= 18))
					{
						uint16 nCompleteCountLast = pClient->GetUpCompleteSourcesCount();
						uint16 nCompleteCountNew = *(uint16*)(pbytePacketBuf + 16);
						pClient->SetUpCompleteSourcesCount(nCompleteCountNew);
						if (nCompleteCountLast != nCompleteCountNew)
							m_pReqPartFile->CalculateCompleteSources();
					}

					CSafeMemFile	packetStream(128);

					if (pClient->GetUDPVersion() > 3)
					{
						if (m_pReqPartFile->IsPartFile())
							((CPartFile*)m_pReqPartFile)->WritePartStatus(&packetStream);
						else
						{
							uint16	uNull = 0;

							packetStream.Write(&uNull, 2);
						}
					}

					uint16	uWaitingPos = g_eMuleApp.m_pUploadQueue->GetWaitingPosition(pClient);

					packetStream.Write(&uWaitingPos, 2);

					Packet	*pPacket = new Packet(&packetStream, OP_EMULEPROT);

					pPacket->m_eOpcode = OP_REASKACK;
					g_eMuleApp.m_pUploadQueue->AddUpDataOverheadFileRequest(pPacket->m_dwSize);
					SendPacket(pPacket, dwIP, port);
				}
			//	If the client isn't currently in our upload queue...
				else
				{
				//	... and there are less than 50 slots free in our upload queue...
					if ((static_cast<uint32>(g_eMuleApp.m_pUploadQueue->GetWaitingUserCount()) + 50) > g_eMuleApp.m_pGlobPrefs->GetQueueSize())
					{
						Packet		*pPacket = new Packet(OP_QUEUEFULL, 0, OP_EMULEPROT);

						g_eMuleApp.m_pUploadQueue->AddUpDataOverheadFileRequest(pPacket->m_dwSize);
						SendPacket(pPacket, dwIP, port);
					}
				}
				break;
			}
			case OP_QUEUEFULL:
			{
				g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(size);

				CUpDownClient		*pClient = g_eMuleApp.m_pDownloadQueue->GetDownloadClientByIP_UDP(dwIP, port);

				if (pClient != NULL)
				{
					pClient->SetRemoteQueueFull(true);
					pClient->UDPReaskACK(0);
				}
			//	ASSERT (false); //Need to double check these asserts to make sure it's running well.
				break;
			}
			case OP_REASKACK:
			{
				g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadFileRequest(size);

				CUpDownClient		*pClient;

				if ((size >= 2) && ((pClient = g_eMuleApp.m_pDownloadQueue->GetDownloadClientByIP_UDP(dwIP, port)) != NULL))
				{
					CSafeMemFile packetStream(pbytePacketBuf, size);

					if ((size >= 4) && (pClient->GetUDPVersion() > 3))
					{
						pClient->ProcessFileStatus(&packetStream, pClient->m_pReqPartFile, true);
					}

					uint16		uRank;

					packetStream.Read(&uRank, 2);

					pClient->SetRemoteQueueFull(false);
					pClient->UDPReaskACK(uRank);
				}
			//	else
			//	ASSERT (false);//Need to double check these asserts to make sure it's running well.
				break;
			}
			case OP_FILENOTFOUND:
			{
				g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadFileRequest(size);

				CUpDownClient		*pClient = g_eMuleApp.m_pDownloadQueue->GetDownloadClientByIP_UDP(dwIP, port);

				if (pClient != NULL)
				{
					pClient->UDPReaskFNF();
				}
				else
				//	ASSERT (false); //Need to double check these asserts to make sure it's running well.
					break;
			}
			default:
			return false;
		}
		return true;
	}
	catch(CFileException* error)
	{
		OUTPUT_DEBUG_TRACE();
		error->Delete();
		AddDebugLogLine(RGB_LOG_ERROR_TXT ERR_UDP_MISCONFIGURED _T(" - CFileException"));
	}
	catch(CMemoryException* error)
	{
		OUTPUT_DEBUG_TRACE();
		error->Delete();
		AddDebugLogLine(RGB_LOG_ERROR_TXT ERR_UDP_MISCONFIGURED _T(" - CMemoryException"));
	}
	catch(CString error)
	{
		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(RGB_LOG_ERROR_TXT ERR_UDP_MISCONFIGURED _T(" - ") + error);
	}
	catch (...)
	{
		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(RGB_LOG_ERROR_TXT ERR_UDP_MISCONFIGURED);
	}
	return false;
}

void CClientUDPSocket::OnSend(int nErrorCode)
{
	EMULE_TRY
	if (nErrorCode)
	{
		return;
	}
	m_bWouldBlock = false;
	while (m_controlPacketQueue.GetHeadPosition() != 0 && !IsBusy())
	{
		UDPPack * cur_packet = m_controlPacketQueue.GetHead();
		char* pcSendBuffer = new char[cur_packet->packet->m_dwSize + 2];
		memcpy2(pcSendBuffer, cur_packet->packet->GetUDPHeader(), 2);
		memcpy2(pcSendBuffer + 2, cur_packet->packet->m_pcBuffer, cur_packet->packet->m_dwSize);
		if (!SendTo(pcSendBuffer, cur_packet->packet->m_dwSize + 2, cur_packet->dwIP, cur_packet->nPort))
		{
			m_controlPacketQueue.RemoveHead();
			delete cur_packet->packet;
			delete cur_packet;
		}
		delete[] pcSendBuffer;
	}
	EMULE_CATCH
}

int CClientUDPSocket::SendTo(char* lpBuf, int nBufLen, uint32 dwIP, uint16 nPort)
{
	EMULE_TRY
	in_addr host;
	host.S_un.S_addr = dwIP;
//	By Maverick Added CClientUDPSocketBaseClass
//	to make it easier to change BaseSocket
	CString s(inet_ntoa(host));
	uint32 result = CClientUDPSocketBaseClass::SendTo(lpBuf, nBufLen, nPort, s);
	if (result == (uint32)SOCKET_ERROR)
	{
		uint32 error = GetLastError();
		if (error == WSAEWOULDBLOCK)
		{
			m_bWouldBlock = true;
			return -1;
		}
		else
		{
			/*Barry - Ignore assert
						ASSERT ( false );
			*/
			return -1;
		}
	}
	EMULE_CATCH
	return 0;
}


bool CClientUDPSocket::SendPacket(Packet* packet, uint32 dwIP, uint16 nPort)
{
	EMULE_TRY
	UDPPack * newpending = new UDPPack;
	newpending->dwIP = dwIP;
	newpending->nPort = nPort;
	newpending->packet = packet;
	if (IsBusy())
	{
		m_controlPacketQueue.AddTail(newpending);
		return true;
	}
	char* pcSendBuffer = new char[packet->m_dwSize + 2];
	memcpy2(pcSendBuffer, packet->GetUDPHeader(), 2);
	memcpy2(pcSendBuffer + 2, packet->m_pcBuffer, packet->m_dwSize);
	if (SendTo(pcSendBuffer, packet->m_dwSize + 2, dwIP, nPort))
	{
		m_controlPacketQueue.AddTail(newpending);
	}
	else
	{
		delete newpending->packet;
		delete newpending;
	}
	delete[] pcSendBuffer;
	return true;
	EMULE_CATCH
	return false;
}

bool CClientUDPSocket::Create()
{
	EMULE_TRY
	if (g_eMuleApp.m_pGlobPrefs->GetUDPPort())
	//	By Maverick Added CClientUDPSocketBaseClass
	//	to make it easier to change BaseSocket
		return CClientUDPSocketBaseClass::Create(g_eMuleApp.m_pGlobPrefs->GetUDPPort(), SOCK_DGRAM, FD_READ);
	else
		return true;
	EMULE_CATCH
	return false;
}
