// XML.cpp: implementation of the CXmlTask class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "../EmEngine.h"
#include "XML.h"

CXmlTask::CXmlTask()
{
	m_ulId = 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CString CXmlTask::PrepareResult()
{
	EMULE_TRY
	bool bOk = false;

	XmlDoc spDoc = XmlCreateDocument();
	if(spDoc)
	{
		XmlElement spElem = XmlCreateElement(spDoc, _T("result"));
		if(spElem)
			bOk = XmlSetAttribute(spElem, _T("id"), m_ulId);
	}

	CString sXml = (LPCTSTR)(spDoc->xml);

	if (!bOk || sXml.IsEmpty())
		throw CString(_T("Can't create XML"));
	TRACE("RETURN: %s (XML)\n", sXml);
	return sXml;
	EMULE_CATCH
	return _T("");
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Removing old definitions
#undef BEGIN_XML_CMD
#undef XML_PARAM_STRING
#undef XML_PARAM_ULONG
#undef END_XML_CMD

// Defining LoadParams function
#define BEGIN_XML_CMD(name)		bool CXml_##name::LoadParams(XmlElement spElem) { 
#define XML_PARAM_STRING(var)		m_s##var = XmlGetAttributeStr(spElem, _T(#var));
#define XML_PARAM_ULONG(var)		m_ul##var = XmlGetAttributeLong(spElem, _T(#var));
#define END_XML_CMD				return true; }
#include "XMLMsgs.h"

//////////////////////////////////////////////////////////////////////
// Parse XML command
// Each call to this function should be in separate thread,
// cause it won't return until command execution is finished.
// E.g. if you command to connect to server, it will return
// immediately - cause connection task is asynchronous,
// but if you will demand file or client list, it will wait until
// list is populated and only then return.
void CEmEngine::ParseXmlCommand(CString sXml)
{
	try
	{
		XmlDoc spDoc = XmlLoadDocumentFromStr(sXml);
		if(spDoc == NULL)
			throw CString(_T("Bad XML"));

		// Get basic parameters - <cmd name="xxx" id="nnn" ... />
		XmlElement spElem = spDoc->selectSingleNode(_T("./cmd"));
		if(spElem == NULL)
			throw CString(_T("Can't find <cmd> tag"));
		CString sCmd = XmlGetAttributeStr(spElem, _T("name"));
		if(sCmd.IsEmpty())
			throw CString(_T("Error reading \"name\" parameter of <cmd> tag"));
		ULONG ulId = XmlGetAttributeLong(spElem, _T("id"), 0);
		if(!ulId)
			throw CString(_T("Error reading \"id\" parameter of <cmd> tag"));

		CXmlTask *pTask = NULL;

		// Removing old definitions
		#undef BEGIN_XML_CMD
		#undef XML_PARAM_STRING
		#undef XML_PARAM_ULONG
		#undef END_XML_CMD

		// Switch by command name, creating appropriate task and load its parameters
		#define BEGIN_XML_CMD(name)		if(!sCmd.CompareNoCase(_T(#name))) pTask = new CXml_##name;
		#define XML_PARAM_STRING(var)
		#define XML_PARAM_ULONG(var)
		#define END_XML_CMD
		#include "XMLMsgs.h"

		if(pTask)
		{
			pTask->m_ulId = ulId;
			if(!pTask->LoadParams(spElem)) 
				delete pTask;
			else
				m_stMainTP.Push(pTask);
		}
	}
	catch (CString &obj)
	{
		// should return <result id="0" error="..." />
		TRACE("ERROR: (XML) %s\n", obj);
		ASSERT(FALSE);
	}
	catch(...)
	{
		// should return <result id="0" error="bad_xml errorcode" />
		TRACE("ERROR: (XML) bad xml\n");
		ASSERT(FALSE);
	}
}
