//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "SearchList.h"
#include "QArray.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//SyruS CQArray-Sorting operators for SearchFileStruct
bool operator > (SearchFileStruct &first, SearchFileStruct &second)
{
	return (first.m_strIndex.CompareNoCase(second.m_strIndex) > 0);
}

bool operator < (SearchFileStruct &first, SearchFileStruct &second)
{
	return (first.m_strIndex.CompareNoCase(second.m_strIndex) < 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CSearchFile::CSearchFile(CFile &in_stream,uint32 dwSearchID,CUpDownClient *pClient,LPCTSTR pszDirectory)
{
	EMULE_TRY

	m_nSearchID = dwSearchID;
	in_stream.Read(&m_fileHash,16);					// <filehash 16>
	in_stream.Read(&m_dwClientIP,4);				// <clientip 4>
	in_stream.Read(&clientport,2);					// <clientport 2>

	uint32		tagcount;

	in_stream.Read(&tagcount,4);					// <tagcount 4>
	for (int i = 0;i != tagcount; i++)
	{
		CFileTag		*pFileTag = new CFileTag(in_stream);	// (tag) * tagcount

		m_tagArray.Add(pFileTag);
	}

	CString strFileName = GetStrTagValue(FT_FILENAME);

	if (!strFileName.IsEmpty())
		SetFileName(strFileName);

	SetFileSize(GetIntTagValue(FT_FILESIZE));

//	Modify some tags for easier and correct future processing
	if (GetFileSize() <= PARTSIZE)
	{
		SetIntTagValue(FT_COMPLETE_SOURCES, GetSourceCount());
		SetIntTagValue(FT_LASTSEENCOMPLETE, 0);	//currently available
	}
	else if (GetCompleteSourceCount() != 0)
		SetIntTagValue(FT_LASTSEENCOMPLETE, 0);	//currently available
	else
		AddExistentIntTag(FT_LASTSEENCOMPLETE, 1);	//modify value to distinguish from 'available'

	if (pClient){		//netwolf 20.05.03 enhanced source finding (bluecow)
		// To be capable of adding the client from which we got that search result
		// as a source, we have to explicitly store that client's data in the search result.
		// NOTE: Do *NOT* store the 'pClient' ptr here! At the time when using the 'toadd'
		// struct, the 'pClient' may have been deleted (just if the user has waited too long before
		// using one of the client's search results) -> Crash!
		m_eType = SFT_CLIENT;

		// Explicitly overwrite the already available client's IP, Port with the already validated UserID and Port.
		// So, we have proper (low)UserID+Server pair.
		m_dwClientIP = pClient->GetUserID();
		clientport = pClient->GetUserPort();

		m_nClientServerIP = pClient->GetServerIP();
		m_nClientServerPort = pClient->GetServerPort();
	}
	else
	{
		m_eType = SFT_SERVER;
		m_nClientServerIP = 0;
		m_nClientServerPort = 0;
	}									// netwolf END

	m_strSearchFileDirectory = pszDirectory ? pszDirectory : _T("");

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CSearchFile::~CSearchFile()
{
	EMULE_TRY
	for (int i = 0; i < m_tagArray.GetSize();i++)
		safe_delete(m_tagArray[i]);
	m_tagArray.RemoveAll();
	m_tagArray.SetSize(0);
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint32 CSearchFile::GetIntTagValue(byte tagname)
{
	EMULE_TRY
	for (int i = 0; i < m_tagArray.GetSize(); i++){
		if (m_tagArray[i]->GetSpecialTag() == tagname)
			return m_tagArray[i]->GetIntValue();
	}
	EMULE_CATCH
	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	GetIntTagValue() returns true if tag was found (in this case pdwOut will contain a valid value on exit)
bool CSearchFile::GetIntTagValue(byte tagname, uint32 *pdwOut)
{
	EMULE_TRY
	for (int i = 0; i < m_tagArray.GetSize(); i++)
	{
		if (m_tagArray[i]->GetSpecialTag() == tagname)
		{
			*pdwOut = m_tagArray[i]->GetIntValue();
			return true;
		}
	}
	EMULE_CATCH
	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchFile::SetIntTagValue(EnumFileTags tagname, uint32 dwVal)
{
	EMULE_TRY

	for (int i = 0; i < m_tagArray.GetSize(); i++)
	{
		if (m_tagArray[i]->GetSpecialTag() == tagname)
		{
			m_tagArray[i]->SetIntValue(dwVal);
			return;
		}
	}
//	if tag was not found, add it
	CFileTag*	pTag = new CFileTag(tagname, dwVal);
	m_tagArray.Add(pTag);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CString CSearchFile::GetStrTagValue(byte tagname)
{
	EMULE_TRY
	for (int i = 0; i < m_tagArray.GetSize(); i++){
		if (m_tagArray[i]->GetSpecialTag() == tagname)
			return m_tagArray[i]->GetStringValue();
	}
	EMULE_CATCH
	return _T("");
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchFile::AddExistentIntTag(byte tagname, uint32 dwInc)
{
	EMULE_TRY
	for (int i = 0; i < m_tagArray.GetSize(); i++)
	{
		if (m_tagArray[i]->GetSpecialTag() == tagname)
		{
			m_tagArray[i]->SetIntValue(m_tagArray[i]->GetIntValue() + dwInc);
			break;
		}
	}
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchFile::AddCompleteSources(uint32 in_dwCount)
{
	EMULE_TRY
	for (int i = 0; i < m_tagArray.GetSize(); i++)
	{
		if (m_tagArray[i]->GetSpecialTag() == FT_COMPLETE_SOURCES)
		{
			m_tagArray[i]->SetIntValue(m_tagArray[i]->GetIntValue() + in_dwCount);
			return;
		}
	}
//	if tag was not found, add it
	CFileTag*	pTag = new CFileTag(FT_COMPLETE_SOURCES, in_dwCount);
	m_tagArray.Add(pTag);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchFile::UpdateLastSeenComplete(uint32 dwSeconds)
{
	EMULE_TRY
	for (int i = 0; i < m_tagArray.GetSize(); i++)
	{
		if (m_tagArray[i]->GetSpecialTag() == FT_LASTSEENCOMPLETE)
		{
			if (dwSeconds < m_tagArray[i]->GetIntValue())
				m_tagArray[i]->SetIntValue(dwSeconds);
			return;
		}
	}
//	if tag was not found, add it
	CFileTag*	pTag = new CFileTag(FT_LASTSEENCOMPLETE, dwSeconds);
	m_tagArray.Add(pTag);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CSearchList::CSearchList()
{
	m_pctlSearchList = NULL;
	m_bMobilMuleSearch = false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CSearchList::~CSearchList()
{
	Clear();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchList::Clear()
{
	EMULE_TRY

	while (!list.IsEmpty())
	{
		delete list.RemoveHead();
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchList::RemoveResults(uint32 nSearchID)
{
	EMULE_TRY
	// this will not delete the item from the window, make sure your code does it if you call this
	ASSERT( m_pctlSearchList );

	POSITION		pos1, pos2;
	CSearchFile	   *pSearchFile;

	for (pos1 = list.GetHeadPosition();( pos2 = pos1 ) != NULL; )
	{
		pSearchFile = list.GetNext(pos1);
		if (pSearchFile->GetSearchID() == nSearchID)
		{
			list.RemoveAt(pos2);
			delete pSearchFile;
		}
	}
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchList::ShowResults(uint32 nSearchID)
{
	EMULE_TRY
	ASSERT( m_pctlSearchList );
	m_pctlSearchList->SetRedraw(false);

	CSearchFile	   *pSearchFile;

	for (POSITION pos = list.GetHeadPosition(); pos != NULL; )
	{
		pSearchFile = list.GetNext(pos);
		if( pSearchFile->GetSearchID() == nSearchID )
		{
			m_pctlSearchList->AddResult(pSearchFile);
		}
	}
	m_pctlSearchList->SetRedraw(true);
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchList::RemoveResults(CSearchFile *todel)
{
	EMULE_TRY

	POSITION remove_pos = list.Find(todel);
	if (remove_pos != NULL)
	{
		g_eMuleApp.m_pdlgEmule->m_dlgSearch.m_ctlSearchList.RemoveResult( todel );
		list.RemoveAt(remove_pos);
		delete todel;
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchList::NewSearch(CSearchListCtrl* in_wnd, CString resTypes, uint16 nSearchID, bool MobilMuleSearch)
{
	EMULE_TRY
	if(in_wnd)
		m_pctlSearchList = in_wnd;

	m_strResultType = resTypes;
	m_dwCurrentSearchCount = nSearchID;
	m_bMobilMuleSearch = MobilMuleSearch;

	foundFilesCount.SetAt(nSearchID,0);
	foundSourcesCount.SetAt(nSearchID,0);
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint16 CSearchList::ProcessSearchAnswer(char *pPacketBuffer, uint32 dwSize, bool* pbIsMoreResultsAvailable)
{
	EMULE_TRY

	if ( g_eMuleApp.m_pdlgEmule->m_dlgSearch.IsLastSearchCanceled() && 
		 !g_eMuleApp.m_pdlgEmule->m_dlgSearch.IsMoreEnabled() )
		return 0;

	CSafeMemFile		packetStream(reinterpret_cast<BYTE*>(pPacketBuffer),dwSize,0);

	uint32		dwNumResults;
	packetStream.Read(&dwNumResults,4);		// <numresults 4>

	for (uint32 i = 0; i < dwNumResults; i++)
	{
		CSearchFile		*pSearchFile = new CSearchFile(packetStream, m_dwCurrentSearchCount);

		AddToList(pSearchFile, false);
	}
#ifdef OLD_SOCKETS_ENABLED
	if (m_bMobilMuleSearch)
	{
		g_eMuleApp.m_pMMServer->SearchFinished(false);
	}
#endif OLD_SOCKETS_ENABLED
	m_bMobilMuleSearch = false;

	if (pbIsMoreResultsAvailable != NULL)
	{
		*pbIsMoreResultsAvailable = false;
		if (packetStream.GetLength() - packetStream.GetPosition() == 1)
		{
			byte	byteMore;

			packetStream.Read(&byteMore, 1);
			if (byteMore == 0x01)
			{
				*pbIsMoreResultsAvailable = true;
			}
		}
	}

	packetStream.Close();

	EMULE_CATCH

	return GetResultCount();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchList::ProcessSharedFileListAnswer(char *pPacketBuffer, uint32 dwSize, CUpDownClient *pClient, LPCTSTR pszDirectory, bool bFirstDir)
{
	EMULE_TRY

	CSafeMemFile		packetStream(reinterpret_cast<BYTE*>(pPacketBuffer),dwSize,0);

	uint32				dwNumResults;

	packetStream.Read(&dwNumResults,4);		// <numresults 4>

	uint32				dwMySearchID = reinterpret_cast<uint32>(pClient);

	if (bFirstDir)
	{
		g_eMuleApp.m_pdlgEmule->m_dlgSearch.DeleteSearch(dwMySearchID);
		g_eMuleApp.m_pdlgEmule->m_dlgSearch.CreateNewTab(pClient->GetUserName(), dwMySearchID);
		foundFilesCount.SetAt(dwMySearchID,0);
	}

	for (int i = 0; i != dwNumResults; i++)
	{
		CSearchFile		*pSearchFile = new CSearchFile(packetStream, dwMySearchID, pClient, pszDirectory);

		AddToList(pSearchFile, true);
	}
#ifdef OLD_SOCKETS_ENABLED
	if (m_bMobilMuleSearch)
	{
		g_eMuleApp.m_pMMServer->SearchFinished(false);
	}
#endif OLD_SOCKETS_ENABLED
	m_bMobilMuleSearch = false;
	packetStream.Close();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint16 CSearchList::ProcessUDPSearchAnswer(byte* pbytePacketBuf, uint32 size)
{
	EMULE_TRY

	if ( g_eMuleApp.m_pdlgEmule->m_dlgSearch.IsLastSearchCanceled() && 
		 !g_eMuleApp.m_pdlgEmule->m_dlgSearch.IsMoreEnabled() )
		return 0;

	CSafeMemFile	packet(pbytePacketBuf,size,0);
	CSearchFile	   *toadd = new CSearchFile(packet, m_dwCurrentSearchCount);

	AddToList(toadd);
	packet.Close();

	EMULE_CATCH
	return GetResultCount();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CSearchList::AddToList(CSearchFile* pAddedFile, bool bClientResponse)
{
	EMULE_TRY

	CString		thisType = pAddedFile->GetFileTypeString(true);

	if ( (!bClientResponse &&
		!( thisType == m_strResultType
			|| m_strResultType == GetResString(IDS_SEARCH_ANY)
			|| m_strResultType == GetResString(IDS_SEARCH_DOC)
			|| (m_strResultType == GetResString(IDS_SEARCH_PRG) && thisType == GetResString(IDS_SEARCH_ARC))) )
		|| ( g_eMuleApp.m_pdlgEmule->m_dlgSearch.IsDocumentSearch(pAddedFile->GetSearchID())
			&& thisType != GetResString(IDS_SEARCH_DOC) )
		|| (!(pAddedFile->GetIntTagValue(FT_FILESIZE)) || pAddedFile->GetFileName().IsEmpty()) )
	{
		delete pAddedFile;
		return false;
	}

	CString	sNotSearch = g_eMuleApp.m_pdlgEmule->m_dlgSearch.GetNotSearch(pAddedFile->GetSearchID());

	if (!sNotSearch.IsEmpty())
	{
		CString	sFileName = pAddedFile->GetFileName();

		sFileName.MakeLower();
		if (sFileName.Find(sNotSearch) != -1)
		{
			delete pAddedFile;
			return false;
		}
	}

	CSearchFile	   *pSearchFile;

	for (POSITION pos = list.GetHeadPosition(); pos != NULL; )
	{
		pSearchFile = list.GetNext(pos);

		if ( (!md4cmp(pAddedFile->GetFileHash(),pSearchFile->GetFileHash())) && pSearchFile->GetSearchID() ==  pAddedFile->GetSearchID())
		{
			uint32	dwTmp, dwAvailableSources = pAddedFile->GetIntTagValue(FT_SOURCES);

		//	Add complete sources tag only if it exists
			if (pAddedFile->GetIntTagValue(FT_COMPLETE_SOURCES, &dwTmp))
			{
				pSearchFile->AddCompleteSources(dwTmp);
				pSearchFile->SetIntTagValue(FT_LASTSEENCOMPLETE, 0);	//currently available
			}
			else if (pAddedFile->GetIntTagValue(FT_LASTSEENCOMPLETE, &dwTmp))
			{
			//	FT_LASTSEENCOMPLETE shouldn't be sent together with FT_COMPLETE_SOURCES
				pSearchFile->UpdateLastSeenComplete(dwTmp);
			}

			pSearchFile->AddExistentIntTag(FT_SOURCES, dwAvailableSources);
			AddResultCount(pSearchFile->GetSearchID(), pAddedFile->GetFileHash(), dwAvailableSources);

			if (m_pctlSearchList && !m_bMobilMuleSearch)
				m_pctlSearchList->UpdateChangingColumns(pSearchFile);

			delete pAddedFile;
			return true;
		}
	}

	if (list.AddTail(pAddedFile))
	{
		uint16 tempValue;
		foundFilesCount.Lookup(pAddedFile->GetSearchID(),tempValue);
		foundFilesCount.SetAt(pAddedFile->GetSearchID(),tempValue+1);

		AddResultCount(pAddedFile->GetSearchID(), pAddedFile->GetFileHash(), pAddedFile->GetIntTagValue(FT_SOURCES));
	}
	if (m_pctlSearchList && !m_bMobilMuleSearch)
		m_pctlSearchList->AddResult(pAddedFile);

	return true;

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchList::AddResultCount(uint32 dwSearchID, const uchar* hash, uint32 dwCount)
{
	//	Do not count already available or downloading files for the search result limit
	if ( g_eMuleApp.m_pDownloadQueue->GetFileByID(hash) == NULL
		&& g_eMuleApp.m_pSharedFilesList->GetFileByID(hash) == NULL )
	{
		uint16	uOldSourcesCount = 0;

		foundSourcesCount.Lookup(dwSearchID,uOldSourcesCount);
		foundSourcesCount.SetAt(dwSearchID,uOldSourcesCount + dwCount);
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint16 CSearchList::GetResultCount(uint32 dwSearchID)
{
	EMULE_TRY
	uint16	uSourcesCount = 0;

	VERIFY( foundSourcesCount.Lookup(dwSearchID, uSourcesCount) );

	return uSourcesCount;
	EMULE_CATCH
	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint16 CSearchList::GetResultCount()
{
	return GetResultCount(m_dwCurrentSearchCount);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CString CSearchList::GetWebList(const CString &strLinePattern, int iSortBy, bool bAscending, bool bShowColumn1, bool bShowColumn2, bool bShowColumn3, bool bShowColumn4, bool bShowColumn5) const
{
	EMULE_TRY

#ifdef _DEBUG
	DWORD dwStart = ::GetTickCount();
	DWORD dwSortTime;
#endif

	CSearchFile* pFile;
	SearchFileStruct structFile;
	CQArray<SearchFileStruct, SearchFileStruct> SearchFileArray;

	for (POSITION pos = list.GetHeadPosition(); pos != NULL; )
	{
		pFile = list.GetNext(pos);

		if (pFile == NULL || pFile->GetFileSize() == 0 || pFile->GetFileName().IsEmpty())
			continue;

		structFile.m_strFileName = pFile->GetFileName();
		structFile.m_strFileType = GetFileTypeForWebServer(structFile.m_strFileName);
		structFile.m_strFileHash = HashToString(pFile->GetFileHash());
		structFile.m_uSourceCount = pFile->GetSourceCount();
		structFile.m_dwCompleteSourceCount = pFile->GetCompleteSourceCount();
		structFile.m_uFileSize = pFile->GetFileSize();
		structFile.m_strFakeCheck = g_eMuleApp.m_pFakeCheck->GetFakeCheckComment(structFile.m_strFileHash, structFile.m_uFileSize);

		switch (iSortBy)
		{
			case 0:
				structFile.m_strIndex = structFile.m_strFileName;
				break;
			case 1:
				structFile.m_strIndex.Format(_T("%10u"), structFile.m_uFileSize);
				break;
			case 2:
				structFile.m_strIndex = structFile.m_strFileHash;
				break;
			case 3:
				structFile.m_strIndex.Format(_T("%09u"), structFile.m_uSourceCount);
				break;
			case 4:
				structFile.m_strIndex = structFile.m_strFakeCheck;
				break;
			case 5:
				structFile.m_strIndex = structFile.m_strFileType;
				break;
			default:
				structFile.m_strIndex.Empty();
		}
		SearchFileArray.Add(structFile);
	}

	SearchFileArray.QuickSort(bAscending);

#ifdef _DEBUG
	DWORD dwEnd = ::GetTickCount();
	dwSortTime = dwEnd - dwStart;
#endif

	uchar aFileHash[16];
	uchar nRed, nGreen, nBlue;
	CKnownFile* sameFile;
	CString strOverlayImage;
	CString strSourcesImage;
	CString strColorPrefix;
	CString strColorSuffix = _T("</font>");
	CString strSources;
	CString strFilename;
	CString strTemp, strTemp2;
	CString strOutput = _T("");

	for (uint16 i = 0; i < SearchFileArray.GetCount(); ++i)
	{
		structFile = SearchFileArray.GetAt(i);
		nRed = nGreen = nBlue = 255;
		DecodeBase16(structFile.m_strFileHash, 32, aFileHash, ARRSIZE(aFileHash));

		sameFile = g_eMuleApp.m_pSharedFilesList->GetFileByID(aFileHash);
		if (sameFile == NULL)
			sameFile = g_eMuleApp.m_pDownloadQueue->GetFileByID(aFileHash);

		if (sameFile == NULL)
		{
			strOverlayImage = _T("none");
		}
		else
		{
			if (sameFile->IsPartFile())
			{
				strOverlayImage = _T("jumpstart");
				nBlue = 0;
				nRed = 0;
			}
			else
			{
				strOverlayImage = _T("release");
				nBlue = 0;
			}
		}
		strColorPrefix.Format(_T("<font color=#%02x%02x%02x>"), nRed, nGreen, nBlue);

		if (structFile.m_uSourceCount < 5)
			strSourcesImage = _T("0");
		else if (structFile.m_uSourceCount > 4 && structFile.m_uSourceCount < 10)
			strSourcesImage = _T("5");
		else if (structFile.m_uSourceCount > 9 && structFile.m_uSourceCount < 25)
			strSourcesImage = _T("10");
		else if (structFile.m_uSourceCount > 24 && structFile.m_uSourceCount < 50)
			strSourcesImage = _T("25");
		else
			strSourcesImage = _T("50");

		strSources.Format(_T("%u(%u)"), structFile.m_uSourceCount, structFile.m_dwCompleteSourceCount);
		strFilename = structFile.m_strFileName;
		strFilename.Replace(_T("'"),_T("\\'"));

		strTemp2.Format(_T("ed2k://|file|%s|%u|%s|/"),
			strFilename, structFile.m_uFileSize, structFile.m_strFileHash);

		strTemp.Format(strLinePattern,
			strSourcesImage, strTemp2, strOverlayImage,
			structFile.m_strFileType,
			(bShowColumn1) ? strColorPrefix + StringLimit(structFile.m_strFileName,70) + strColorSuffix : _T(""),
			(bShowColumn2) ? strColorPrefix + CastItoXBytes(structFile.m_uFileSize) + strColorSuffix : _T(""),
			(bShowColumn3) ? strColorPrefix + structFile.m_strFileHash + strColorSuffix : _T(""),
			(bShowColumn4) ? strColorPrefix + strSources + strColorSuffix : _T(""),
			(bShowColumn5) ? strColorPrefix + structFile.m_strFakeCheck + strColorSuffix : _T(""),
			structFile.m_strFileHash
			);
		strOutput.Append(strTemp);
	}

#ifdef _DEBUG
	g_eMuleApp.AddDebugLogLine(_T("WebServer: Searchlist with %u elements sorted in %u ms, output generated in %u ms"), SearchFileArray.GetSize(), dwSortTime, ::GetTickCount()-dwEnd);
#endif

	return strOutput;

	EMULE_CATCH
	return _T("");
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSearchList::AddFileToDownloadByHash(uchar* hash,EnumCategories eCatID)
{
	EMULE_TRY

	CSearchFile		*pSearchFile;

	for (POSITION pos = list.GetHeadPosition(); pos != NULL; )
	{
		pSearchFile = list.GetNext(pos);
		if (!md4cmp(hash,pSearchFile->GetFileHash()))
		{
			g_eMuleApp.m_pDownloadQueue->AddSearchToDownload(pSearchFile, eCatID, g_eMuleApp.m_pGlobPrefs->StartDownloadPaused());
			break;
		}
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CSearchFile* CSearchList::DetachNextFile(uint32 nSearchID)
{
// mobilemule
	EMULE_TRY
	// the files are NOT deleted, make sure you do this if you call this function
	// find, removes and returns the searchresult with most Sources
	uint32			nHighSource = 0;
	POSITION		resultpos = 0;
	CSearchFile	   *pSearchFile;

	for (POSITION pos = list.GetHeadPosition(); pos != NULL; )
	{
		pSearchFile = list.GetNext(pos);
		if (pSearchFile->GetSearchID() == nSearchID)
		{
			if (pSearchFile->GetIntTagValue(FT_SOURCES) > nHighSource)
			{
				nHighSource = pSearchFile->GetIntTagValue(FT_SOURCES);
				resultpos = pos;
			}
		}
	}
	if (resultpos == NULL)
	{
		ASSERT ( false );
		return NULL;
	}
	CSearchFile* result = list.GetAt(resultpos);
	list.RemoveAt(resultpos);
	return result;
	EMULE_CATCH
	return NULL;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
