//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include "stdafx.h"
#include "emule.h"
#include "packets.h"
#include "UDPSocket.h"
#include "otherfunctions.h"
#include "otherstructs.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#ifdef OLD_SOCKETS_ENABLED

#define	ERR_UDP_MISCONFIGURED _T("Error while processing incoming UDP Packet (most likely a misconfigured server)")

CUDPSocketWnd::CUDPSocketWnd(){
}

BEGIN_MESSAGE_MAP(CUDPSocketWnd, CWnd)
	ON_MESSAGE(WM_DNSLOOKUPDONE, OnDNSLookupDone)
END_MESSAGE_MAP()

LRESULT CUDPSocketWnd::OnDNSLookupDone(WPARAM wParam,LPARAM lParam)
{
	EMULE_TRY

	if(m_pOwner)
		m_pOwner->DnsLookupDone(wParam,lParam);
	return true;

	EMULE_CATCH

	return false;
};
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CUDPSocket::CUDPSocket(CServerConnect* in_serverconnect)
{
	m_hWndResolveMessage = NULL;
	m_pcSendBuffer = 0;
	m_pServer = 0;
	m_pServerConnect = in_serverconnect;
	DnsTaskHandle = 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CUDPSocket::~CUDPSocket()
{
	EMULE_TRY

	if (m_pServer)
		delete m_pServer;
	if (m_pcSendBuffer)
		delete[] m_pcSendBuffer;

	m_udpwnd.DestroyWindow();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool  CUDPSocket::Create()
{
	EMULE_TRY

	VERIFY( m_udpwnd.CreateEx(0, AfxRegisterWndClass(0),_T("Emule Socket Wnd"),WS_OVERLAPPED, 0, 0, 0, 0, NULL, NULL));
	m_hWndResolveMessage = m_udpwnd.m_hWnd;
	m_udpwnd.m_pOwner = this;

	return CAsyncSocket::Create(0,SOCK_DGRAM,FD_READ);

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CUDPSocket::OnReceive(int nErrorCode)
{
	EMULE_TRY

	byte		abyteBuffer[5000];
	CString		serverbuffer;
	UINT		uiPort;
	int			iLength = ReceiveFrom(abyteBuffer, sizeof(abyteBuffer), serverbuffer, uiPort);

	if ((iLength >= 2) && (abyteBuffer[0] == OP_EDONKEYPROT))	//SOCKET_ERROR = -1
		ProcessPacket(abyteBuffer + 2, iLength - 2, abyteBuffer[1], serverbuffer.GetBuffer(), uiPort);
	
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUDPSocket::ProcessPacket(byte* pbytePacketBuf, uint16 uSize, byte uOpcode, LPCTSTR host, uint16 uPort)
{
	CServer	*pServer = NULL;

	try
	{
		g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadOther(uSize);
		if ((pServer = g_eMuleApp.m_pServerList->GetServerByAddress(host, uPort - 4)) == NULL)
		{
		//	Don't process packet from unknown servers
			return false;
		}
		pServer->ResetFailedCount();
		g_eMuleApp.m_pdlgEmule->m_wndServer.m_ctlServerList.RefreshServer(*pServer);

		switch (uOpcode)
		{
			case OP_GLOBSEARCHRES:
			{
				g_eMuleApp.m_pdlgEmule->m_dlgSearch.AddUDPResult(g_eMuleApp.m_pSearchList->ProcessUDPSearchAnswer(pbytePacketBuf,uSize));
				break;
			}
			case OP_GLOBFOUNDSOURCES:
			{
				CSafeMemFile	sources((BYTE*)pbytePacketBuf,uSize);
				uchar			fileid[16];

				sources.Read(fileid,16);

				CPartFile* 		pPartFile = g_eMuleApp.m_pDownloadQueue->GetFileByID(fileid);

				if (pPartFile)
					pPartFile->AddSources(sources,inet_addr(host),uPort);
				else
					g_eMuleApp.m_pServerList->CancelUDPBroadcast();				

				break;
			}
 			case OP_GLOBSERVSTATRES:
			{
			//	Check minimal allowed size (contain basic information)
				if (uSize < 12)
					return false;

				uint32		dwChallenge;
				
				memcpy2(&dwChallenge,pbytePacketBuf,4);

				if (dwChallenge != pServer->GetChallenge()) 
					return true; 

				uint32		dwCurUsersNumber;
				uint32		dwCurFilesNumber;
				uint32		dwMaxUsersNumber = 0;
				uint32		dwCurSoftFilesLimit = 0;
				uint32		dwCurHardFilesLimit = 0;
				uint32		dwUDPFlags = 0;
				
				memcpy2(&dwCurUsersNumber,pbytePacketBuf+4,4);
				memcpy2(&dwCurFilesNumber,pbytePacketBuf+8,4);

				if (uSize >= 16)
				{
					memcpy2(&dwMaxUsersNumber, pbytePacketBuf+12,4);
				}
				if (uSize >= 24)
				{
					memcpy2(&dwCurSoftFilesLimit, pbytePacketBuf+16,4);
					memcpy2(&dwCurHardFilesLimit, pbytePacketBuf+20,4);
				}
				if (uSize >= 28)
				{
					memcpy2(&dwUDPFlags, pbytePacketBuf+24,4);
				}
				if (pServer)
				{
					pServer->SetPing(::GetTickCount() - pServer->GetLastPinged());
					pServer->SetUserCount(dwCurUsersNumber);
					pServer->SetFileCount(dwCurFilesNumber);
					pServer->SetMaxUsers(dwMaxUsersNumber);
					pServer->SetSoftMaxFiles(dwCurSoftFilesLimit);
					pServer->SetHardMaxFiles(dwCurHardFilesLimit);
					pServer->SetUDPFlags(dwUDPFlags);
					g_eMuleApp.m_pdlgEmule->m_wndServer.m_ctlServerList.RefreshServer(*pServer);
				}
				break;
			}

 			case OP_SERVER_DESC_RES:
			{
				CSafeMemFile srvinfo((BYTE*)pbytePacketBuf,uSize);

				uint16 uStringLen;
				srvinfo.Read(&uStringLen,2);
				CString strName;
				srvinfo.Read(strName.GetBuffer(uStringLen),uStringLen);
				strName.ReleaseBuffer(uStringLen);
				
				srvinfo.Read(&uStringLen,2);
				CString strDesc;
				srvinfo.Read(strDesc.GetBuffer(uStringLen),uStringLen);
				strDesc.ReleaseBuffer(uStringLen);

				if(pServer)
				{
					pServer->SetDescription(strDesc);
					if(!pServer->IsStaticMember())
						pServer->SetListName(strName);
					g_eMuleApp.m_pdlgEmule->m_wndServer.m_ctlServerList.RefreshServer(*pServer);
				}
				break;
			}
			default:
				return false;
		}
		return true;
	}
	catch(CFileException* error)
	{
		CString	strBuff(RGB_LOG_ERROR_TXT ERR_UDP_MISCONFIGURED);

		if (pServer != NULL)
			strBuff.AppendFormat(_T(" %s:%u"), pServer->GetAddress(), pServer->GetPort());

		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(strBuff);
		error->Delete();
	}
	catch(CMemoryException* error)
	{
		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(RGB_LOG_ERROR_TXT ERR_UDP_MISCONFIGURED _T(" - CMemoryException"));
		error->Delete();
	}
	catch(CString error)
	{
		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(RGB_LOG_ERROR_TXT ERR_UDP_MISCONFIGURED _T(" - ") + error);
	}
	catch(...)
	{
		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(RGB_LOG_ERROR_TXT ERR_UDP_MISCONFIGURED);
	}
	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CUDPSocket::AsyncResolveDNS(LPCTSTR lpszHostAddress, UINT nHostPort)
{
	EMULE_TRY

	m_lpszHostAddress = lpszHostAddress;
	m_nHostPort = nHostPort;
	if (DnsTaskHandle)
		WSACancelAsyncRequest(DnsTaskHandle);
	DnsTaskHandle = NULL;
// 	See if we have an IP already
	USES_CONVERSION;
	SOCKADDR_IN sockAddr;
	memzero(&sockAddr, sizeof(sockAddr));
	LPSTR lpszAscii = T2A((LPTSTR)m_lpszHostAddress);
	sockAddr.sin_family = AF_INET;
	sockAddr.sin_addr.s_addr = inet_addr(lpszAscii);
	sockAddr.sin_port = htons((u_short)m_nHostPort);

//	Backup for send socket
	m_SaveAddr = sockAddr;

	if (sockAddr.sin_addr.s_addr == INADDR_NONE)
	{
	//	Resolve hostname "hostname" asynchronously
		memzero(DnsHostBuffer, sizeof(DnsHostBuffer));

		USES_CONVERSION;
		DnsTaskHandle = WSAAsyncGetHostByName( m_hWndResolveMessage,
											   WM_DNSLOOKUPDONE,
											   CT2A(lpszHostAddress),
											   DnsHostBuffer,
											   MAXGETHOSTSTRUCT );

		if (DnsTaskHandle == 0)
		{
			delete[] m_pcSendBuffer;
			m_pcSendBuffer = 0;
			delete m_pServer;
			m_pServer = 0;
#ifdef _DEBUG
			AfxMessageBox(_T("LOOKUPERROR DNSTASKHANDLE = 0"));
#endif
		}
	}
	else
	{
		SendBuffer();
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CUDPSocket::DnsLookupDone(WPARAM wp, LPARAM lp)
{
	EMULE_TRY

	DnsTaskHandle = NULL;

//	An asynchronous database routine completed
	if (WSAGETASYNCERROR(lp) != 0)
	{
		if (m_pcSendBuffer)
			delete[] m_pcSendBuffer;
		m_pcSendBuffer = 0;
		if (m_pServer)
			delete m_pServer;
		m_pServer = 0;
		return;
	}
	if (m_SaveAddr.sin_addr.s_addr == INADDR_NONE)
	{
	//	Get the structure length
		int iBufLen = WSAGETASYNCBUFLEN(lp);
		LPHOSTENT lphost = (LPHOSTENT)malloc(iBufLen);
		memcpy2(lphost, DnsHostBuffer, iBufLen);
		m_SaveAddr.sin_addr.s_addr = ((LPIN_ADDR)lphost->h_addr)->s_addr;
		free(lphost);
	//	Also reset the receive buffer
		memzero(DnsHostBuffer, sizeof(DnsHostBuffer));
	}
	if (m_pServer)
	{
		CServer* update = g_eMuleApp.m_pServerList->GetServerByAddress(m_pServer->GetAddress(),m_pServer->GetPort());
		if (update)
			update->SetID(m_SaveAddr.sin_addr.S_un.S_addr);
		SendBuffer();
	}
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CUDPSocket::SendBuffer()
{
	EMULE_TRY

	if(m_pServer && m_pcSendBuffer)
	{
		SendTo(m_pcSendBuffer,m_dwSendBufLength,(SOCKADDR*)&m_SaveAddr, sizeof(m_SaveAddr));
	}
	safe_delete(m_pServer);
	safe_delete(m_pcSendBuffer);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CUDPSocket::SendPacket(Packet *pPacket,CServer *pHostServer)
{
//	At this point m_pServer & m_pcSendBuffer must be already NULL but it seems that sometime we are not able
//	to get WM_DNSLOOKUPDONE before we send a new packet therefore the existency of the objects will be checked
	safe_delete(m_pServer);
	safe_delete(m_pcSendBuffer);

//	Now we are trying to create a new objects
	try
	{
		m_pServer = new CServer(*pHostServer);
		m_pcSendBuffer = new char[pPacket->m_dwSize+sizeof(UDP_Header_Struct)];
		memcpy2(m_pcSendBuffer,pPacket->GetUDPHeader(),sizeof(UDP_Header_Struct));
		memcpy2(m_pcSendBuffer+sizeof(UDP_Header_Struct),pPacket->m_pcBuffer,pPacket->m_dwSize);
		m_dwSendBufLength = pPacket->m_dwSize+sizeof(UDP_Header_Struct);
		AsyncResolveDNS(m_pServer->GetAddress(),m_pServer->GetPort()+4);
	}
	catch (CException * error)
	{
		OUTPUT_DEBUG_TRACE();
		error->Delete();
		safe_delete(m_pServer);
		safe_delete(m_pcSendBuffer);
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif //OLD_SOCKETS_ENABLED
