//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once

#include "types.h"
#include "opcodes.h"
#include "otherstructs.h"
#include "zlib/zlib.h"

//			CLIENT TO SERVER
//			PACKET CLASS
// TODO some parts could need some work to make it more efficient
class Packet
{
public:
					Packet(EnumProtocols eProtocol = OP_EDONKEYPROT);
					Packet(PacketHeader_Struct *pHeaderStruct); // only used for receiving packets
					Packet(CMemFile *pDataFile,EnumProtocols eProtocol = OP_EDONKEYPROT);
				Packet(char* pPacketPart,uint32 nSize,bool bLast, byte byteFilePriority, bool bFromPF = true); // only used for split packets!
					Packet(EnumOpcodes eOpcode,uint32 iSize = 0,EnumProtocols eProtocol = OP_EDONKEYPROT,bool bFromPF = true);
				Packet(EnumOpcodes eOpcode, uint32 iSize, EnumProtocols eProtocol, byte byteFilePriority, bool bFromPF);
				   ~Packet();
	char		   *GetHeader();
	char		   *GetUDPHeader();
	char		   *GetPacket();
	char		   *DetachPacket();
	uint32			GetRealPacketSize()		{return m_dwSize+sizeof(PacketHeader_Struct);}
	bool			IsSplit()			{return m_bSplit;}
	bool			IsLastSplit()		{return m_bLastSplit;}
	bool			IsFromPF()			{return m_bFromPF;}	//	khaos stats
	byte 			GetFilePriority()	{return m_byteFilePriority;}
	void			PackPacket();
	bool			UnpackPacket(uint32 dwMaxUnpackedSize = 0);

	char		   *m_pcBuffer;
	uint32			m_dwSize;
	EnumOpcodes		m_eOpcode;
	EnumProtocols	m_eProtocol;

private:
	byte		m_byteFilePriority;
	bool			m_bSplit;
	bool			m_bLastSplit;
	bool			m_bFromPF;
	char			m_arrcHead[6];
	char		   *m_pcCompleteBuffer;
	char		   *m_pcTempBuffer;
	bool			m_bPacked;
};

//	Tag Types
#define TAGTYPE_NONE		0
#define TAGTYPE_HASH		0x01
#define TAGTYPE_STRING		0x02
#define TAGTYPE_UINT32		0x03
#define TAGTYPE_FLOAT32		0x04
#define TAGTYPE_BOOL		0x05
#define TAGTYPE_BOOLARRAY	0x06
#define TAGTYPE_BLOB		0x07
#define TAGTYPE_UINT16		0x08
#define TAGTYPE_UINT8		0x09
#define TAGTYPE_BSOB		0x0A
#define TAGTYPE_STR1		0x11
#define TAGTYPE_STR2		0x12
#define TAGTYPE_STR3		0x13
#define TAGTYPE_STR4		0x14
#define TAGTYPE_STR5		0x15
#define TAGTYPE_STR6		0x16
#define TAGTYPE_STR7		0x17
#define TAGTYPE_STR8		0x18
#define TAGTYPE_STR9		0x19
#define TAGTYPE_STR10		0x1A
#define TAGTYPE_STR11		0x1B
#define TAGTYPE_STR12		0x1C
#define TAGTYPE_STR13		0x1D
#define TAGTYPE_STR14		0x1E
#define TAGTYPE_STR15		0x1F
#define TAGTYPE_STR16		0x20

class CTag
{
public:
					CTag(byte byteTagID, uint32 dwValue);
					CTag(byte byteTagID, LPCTSTR strValue);
					CTag(LPCTSTR strName, uint32 dwValue);
					CTag(LPCTSTR strName, LPCTSTR strValue);
					CTag(CFile& file);
	bool			WriteToFile(CFile &file);
	bool			WriteToFile(FILE *pFile);

	byte			GetSpecialTag() const { return m_byteTagID; }
	CString			GetStringValue() const { return m_strValue; }
	CString			GetTagName() const { return m_strName; }
	void			SetTagName(LPCTSTR strName) { m_strName = strName; }
	uint32			GetIntValue() const { return m_dwValue; }
	void			SetIntValue(uint32 dwValue) { m_dwValue = dwValue; }
	byte			GetType() const { return m_byteTagType; }
	void			Clear()
	{
		m_byteTagType = TAGTYPE_NONE;
		m_strName.Empty();
		m_strValue.Empty();
		m_dwValue = 0;
		m_fltValue = 0.0f;
//		m_byteTagID = 0;
	}

protected:
	CString			m_strName;
	CString			m_strValue;
	uint32			m_dwValue;
	float			m_fltValue;
	byte			m_byteTagType;
	byte			m_byteTagID;
};

typedef CTag	CClientTag;
typedef CTag	CFriendTag;
typedef CTag	CEmuleTag;
typedef CTag	CFileTag;
typedef CTag	CServerTag;
